/*
 * ssidsniff - a tool for wireless network reconnaissance
 * Copyright (C) 2002-2007 Kostas Evangelinos <kos@bastard.net>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*
 * $Id: 80211.h,v 1.20 2007-06-28 14:28:01 kos Exp $
 *
 */

#ifndef _80211_H
#define _80211_H

#ifdef HAVE_PCAP_H
#include "pcap.h"
#endif

#define f80211_LINELEN 1024

/* tcpdump extract.h types */
typedef struct {
        u_int16_t       val;
} __attribute__((packed)) unaligned_u_int16_t;

typedef struct {
        u_int32_t       val;
} __attribute__((packed)) unaligned_u_int32_t;

/* This is from the 80211 dissector included with ethereal */
/* IEEE802.11 Frame types */
#define MGT_FRAME            0x00       /* Frame type is management */
#define CONTROL_FRAME        0x01       /* Frame type is control */
#define DATA_FRAME           0x02       /* Frame type is Data */

/* IEEE802.11 Frame subtypes */
#define MGT_ASSOC_REQ        0x00       /* Management - association request        */
#define MGT_ASSOC_RESP       0x01       /* Management - association response       */
#define MGT_REASSOC_REQ      0x02       /* Management - reassociation request      */
#define MGT_REASSOC_RESP     0x03       /* Management - reassociation response     */
#define MGT_PROBE_REQ        0x04       /* Management - Probe request              */
#define MGT_PROBE_RESP       0x05       /* Management - Probe response             */
#define MGT_BEACON           0x08       /* Management - Beacon frame               */
#define MGT_ATIM             0x09       /* Management - ATIM                       */
#define MGT_DISASS           0x0A       /* Management - Disassociation             */
#define MGT_AUTHENTICATION   0x0B       /* Management - Authentication             */
#define MGT_DEAUTHENTICATION 0x0C       /* Management - Deauthentication           */
#define MGT_ACTION           0x0D	      /* Management - Action */
#define CTRL_PS_POLL         0x1A       /* Control - power-save poll               */
#define CTRL_RTS             0x1B       /* Control - request to send               */
#define CTRL_CTS             0x1C       /* Control - clear to send                 */
#define CTRL_ACKNOWLEDGEMENT 0x1D       /* Control - acknowledgement               */
#define CTRL_CFP_END         0x1E       /* Control - contention-free period end    */
#define CTRL_CFP_ENDACK      0x1F       /* Control - contention-free period end/ack */
#define DATA                 0x20       /* Data - Data                             */
#define DATA_CF_ACK          0x21       /* Data - Data + CF acknowledge            */
#define DATA_CF_POLL         0x22       /* Data - Data + CF poll                   */
#define DATA_CF_ACK_POLL     0x23       /* Data - Data + CF acknowledge + CF poll  */
#define DATA_NULL_FUNCTION   0x24       /* Data - Null function (no data)          */
#define DATA_CF_ACK_NOD      0x25       /* Data - CF ack (no data)                 */
#define DATA_CF_POLL_NOD     0x26       /* Data - Data + CF poll (No data)         */
#define DATA_CF_ACK_POLL_NOD 0x27       /* Data - CF ack + CF poll (no data)       */

#define WPA_OUI	"\x00\x50\xF2"

/* Beacon header capability */
#define IEEE80211_CAPINFO_ESS                   0x0001
#define IEEE80211_CAPINFO_IBSS                  0x0002
#define IEEE80211_CAPINFO_CF_POLLABLE           0x0004
#define IEEE80211_CAPINFO_CF_POLLREQ            0x0008
#define IEEE80211_CAPINFO_PRIVACY               0x0010
#define IEEE80211_CAPINFO_SHORT_PREAMBLE        0x0020
#define IEEE80211_CAPINFO_PBCC                  0x0040
#define IEEE80211_CAPINFO_CHNL_AGILITY          0x0080

/* IEEE802.11 tags */
#define TAG_SSID           0x00
#define TAG_SUPP_RATES     0x01
#define TAG_FH_PARAMETER   0x02
#define TAG_DS_PARAMETER   0x03
#define TAG_CF_PARAMETER   0x04
#define TAG_TIM            0x05
#define TAG_IBSS_PARAMETER 0x06
#define TAG_CHALLENGE_TEXT 0x10
#define TAG_EXTENDED_RATES 0x32
#define TAG_VENDOR_SPECIFIC 0xdd
#define TAG_WPA2           0x30

/* radiotap */
enum ieee80211_radiotap_type {
  IEEE80211_RADIOTAP_TSFT = 0,              /* unsigned 64 bit */
  IEEE80211_RADIOTAP_FLAGS = 1,             /* unsigned 8 bit */
  IEEE80211_RADIOTAP_RATE = 2,              /* unsigned 8 bit */  
  IEEE80211_RADIOTAP_CHANNEL = 3,           /* 2x unsigned 16 bit */
  IEEE80211_RADIOTAP_FHSS = 4,              /* 2x 8 bit */
  IEEE80211_RADIOTAP_DBM_ANTSIGNAL = 5,     /* signed 8 bit */
  IEEE80211_RADIOTAP_DBM_ANTNOISE = 6,      /* signed 8 bit */
  IEEE80211_RADIOTAP_LOCK_QUALITY = 7,      /* unsigned 16 bit */
  IEEE80211_RADIOTAP_TX_ATTENUATION = 8,    /* unsigned 16 bit */
  IEEE80211_RADIOTAP_DB_TX_ATTENUATION = 9,
  IEEE80211_RADIOTAP_DBM_TX_POWER = 10,
  IEEE80211_RADIOTAP_ANTENNA = 11,
  IEEE80211_RADIOTAP_DB_ANTSIGNAL = 12,
  IEEE80211_RADIOTAP_DB_ANTNOISE = 13,
  IEEE80211_RADIOTAP_EXT = 31,
};

struct ieee80211_radiotap_header {
  u_int8_t        it_version;     /* set to 0 */
  u_int8_t        it_pad;
  u_int16_t       it_len;         /* entire length */
  u_int32_t       it_present;     /* fields present */
} __attribute__((__packed__));

/* prism2 */
#define PRISM2_LEN 144
#define WLAN_DEVNAMELEN_MAX 16

#define DIDmsg_lnxind_wlansniffrm 0x0041
#define DIDmsg_lnxind_wlansniffrm_hosttime 0x1041
#define DIDmsg_lnxind_wlansniffrm_mactime 0x2041
#define DIDmsg_lnxind_wlansniffrm_channel 0x3041
#define DIDmsg_lnxind_wlansniffrm_rssi 0x4041
#define DIDmsg_lnxind_wlansniffrm_sq 0x5041
#define DIDmsg_lnxind_wlansniffrm_signal 0x6041
#define DIDmsg_lnxind_wlansniffrm_noise 0x7041
#define DIDmsg_lnxind_wlansniffrm_rate 0x8041
#define DIDmsg_lnxind_wlansniffrm_istx 0x9041
#define DIDmsg_lnxind_wlansniffrm_frmlen 0xA041

typedef struct p80211item_uint32 {
  unsigned int did;
  unsigned short status;
  unsigned short len;
  unsigned int data;
} p80211item_uint32_t;

struct prism2_hdr {
  u_int32_t     msgcode;
  u_int32_t     msglen;
  u_int8_t      devname[WLAN_DEVNAMELEN_MAX];
  p80211item_uint32_t      hosttime;
  p80211item_uint32_t     mactime;
  p80211item_uint32_t     channel;
  p80211item_uint32_t     rssi;
  p80211item_uint32_t     sq;
  p80211item_uint32_t     signal;
  p80211item_uint32_t     noise;
  p80211item_uint32_t     rate;
  p80211item_uint32_t     istx;
  p80211item_uint32_t     frmlen;
} __attribute__ ((packed));

typedef struct frame_control_t {
  unsigned version:2;
  unsigned type:2;
  unsigned subtype:4;
  unsigned to_ds:1;
  unsigned from_ds:1;
  unsigned more_frag:1;
  unsigned retry:1;
  unsigned powermgmt:1;
  unsigned more_data:1;
  unsigned wep:1;
  unsigned order:1;
} frame_control_t;

typedef struct f80211_mac_t {
  unsigned char fc[2];
  unsigned char id[2];
  unsigned char address1[6];
  unsigned char address2[6];
  unsigned char address3[6];
  unsigned char seq[2];
} f80211_mac_t;

typedef struct beacon_header_t {
  unsigned char timestamp[8];
  unsigned char interval[2];
  unsigned char capability[2];
} beacon_header_t;

typedef struct f80211_t {
  char ssid[f80211_LINELEN];  
  char dst[f80211_LINELEN];
  char src[f80211_LINELEN];
  char bss[f80211_LINELEN];
  char manufacturer[f80211_LINELEN];
  unsigned int channel;
  unsigned int type;
  unsigned int subtype;
  unsigned int wepcapable;
  unsigned int wepused;
  unsigned int ap;
  unsigned int xrates;
  unsigned int wpa;
  unsigned int wpa2;
  unsigned int interval;
  char signal;
  f80211_mac_t f_mac;
} f80211_t;

typedef enum {
  f80211_UNKNOWN,
  f80211_BEACON,
  f80211_PROBERESP,
  f80211_PROBEREQ,
  f80211_DATA
} f80211_frame_t;
  
int f80211_parse(unsigned char *c, struct pcap_pkthdr *, int, f80211_t *);
int f80211_isbeacon(f80211_t *);
int f80211_isdata(f80211_t *);
int f80211_isprobereq(f80211_t *);
int f80211_isproberesp(f80211_t *);
f80211_frame_t f80211_type(f80211_t *);
int f80211_checkdlt(int dlt);

#endif /* _80211_H */
