/*
 * ssidsniff - a tool for wireless network reconnaissance
 * Copyright (C) 2002-2007 Kostas Evangelinos <kos@bastard.net>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*
 * $Id: ssidsniff.h,v 1.43 2007-06-28 14:28:01 kos Exp $
 *
 */

#ifndef _SSIDSNIFF_H
#define _SSIDSNIFF_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/time.h>
#include <pcap.h>
#include <stdio.h>
#include <unistd.h>

#include "util.h"
#include "xpaudio.h"
#include "80211.h"
#include "hopper.h"
#include "mach.h"

#define DEF_LINELEN 1024
#define DEF_GEIGER_L 128
#define DEF_WNET_ALLOC 64
#define DEF_WRFILE "ssidsniff.cap"

typedef enum {
  CAPTURE_NONE=0, CAPTURE_DATA, CAPTURE_ALL
} capture_mode_t;

typedef struct counter_t {
  unsigned int total;
  unsigned int beacon;
  unsigned int data;
  unsigned int martian;
} counter_t;

typedef struct wnet_client_t {
  struct wnet_client_t *next;
  char src[DEF_LINELEN];
  char manufacturer[DEF_LINELEN];
  int ap;
  unsigned int count;
} wnet_client_t;

typedef struct wnet_t {
  unsigned int serial;
  char bss[DEF_LINELEN];
  char ssid[DEF_LINELEN];
  char dst[DEF_LINELEN];
  int capture;
  int ap;
  int xrates;
  int wpa;
  int wpa2;
  int wepcapable;
  int wepused;
  int channel;
  counter_t count;
  unsigned int pps;
  struct timeval pps_prevts;
  unsigned int pps_delta;
  double freq;
  strings_t strings;
  char signal;
  wnet_client_t *clients;
  struct timeval last_activity;
} wnet_t;

typedef struct ssidsniff_t {
  char device[PCAP_ERRBUF_SIZE];
  char dumpfile[DEF_LINELEN];
  char filter[DEF_LINELEN];
  wnet_t *networks;
  int num_networks;
  int max_networks;
  int geiger;
  int delay;
  int running;
  int capture;
  time_t start;
  xpaudio_t *xpa;
  counter_t globalcount;
  pcap_t *pd;
  int pcap_fd;
  int pcap_dlt;
  pcap_dumper_t *pd_dump;
  hopper_t *hopper;
  int run_hopper;
} ssidsniff_t;

int wnet_clear(wnet_t *);
wnet_t *wnet_add(ssidsniff_t *s, char *key);
wnet_t *wnet_get(ssidsniff_t *s, int which);
wnet_t *wnet_find(ssidsniff_t *s, char *key);
int wnet_record(ssidsniff_t *s);
int field_get_value(int field_id, wnet_t *sample, char *res, int resl);
int pps_update_all(ssidsniff_t *s);

#endif /* _SSIDSNIFF_H */
