/*
 * ssidsniff - a tool for wireless network reconnaissance
 * Copyright (C) 2002-2007 Kostas Evangelinos <kos@bastard.net>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*
 * $Id: xpaudio.h,v 1.5 2007-06-20 01:47:12 kos Exp $
 *
 */

#ifndef _XPAUDIO_H
#define _XPAUDIO_H

#define XPA_LINELEN 1024

#define XPA_RATE (1<<0)
#define XPA_ENCODING (1<<1)
#define XPA_NCHANNELS (1<<2)

#define XPA_ENCODING_LINEAR (1<<1)
#define XPA_ENCODING_ULAW (1<<2)
#define XPA_ENCODING_8BIT (1<<3)
#define XPA_ENCODING_16BIT (1<<4)
#define XPA_ENCODING_SIGNED (1<<5)
#define XPA_ENCODING_UNSIGNED (1<<6)

#define XPA_FRAGSIZE 1000
#define XPA_RATE_DEFAULT 44100
#define XPA_NCHANNELS_DEFAULT 2
#define XPA_ENCODING_DEFAULT (XPA_ENCODING_LINEAR|XPA_ENCODING_16BIT|XPA_ENCODING_UNSIGNED)

#ifdef HAVE_ALSA_ASOUNDLIB_H
#include <alsa/asoundlib.h>
#endif

typedef struct xpaudio_t {
  int fd;
  unsigned int rate;
  int channels;
  int encoding;
  int blksz;
  char device[XPA_LINELEN];
  char buffer[XPA_FRAGSIZE];
#ifdef HAVE_ALSA_ASOUNDLIB_H
  snd_pcm_t *alsa_handle;
  snd_pcm_hw_params_t *hw_params;
#endif
} xpaudio_t ;

xpaudio_t *xpaudio_new(char *dev);
int xpaudio_destroy(xpaudio_t *);
int xpaudio_set(xpaudio_t *xp, unsigned int param, int what);
int xpaudio_get(xpaudio_t *xp, unsigned int param);
int xpaudio_send(xpaudio_t *xp, char *sample, unsigned int samplen);
char *xpaudio_describe(xpaudio_t *xpa);

#endif /* _XPAUDIO_H */
